import{j as e}from"./vendor-tanstack-BpuysabX.js";import{f as o,H as r,bZ as n,cj as s,ct as i}from"./index-CREafGO4.js";const d={READY:e.jsx(s,{size:12,fill:"green"}),NOT_READY:e.jsx(s,{size:12,fill:"red"}),PAUSED:e.jsx(n,{size:12})},g=i("gap-2 px-2 text-foreground/80 font-mono font-light border border-foreground/10 shadow-none text-nowrap",{variants:{status:{READY:"bg-state-completed-100 hover:bg-state-completed-200",NOT_READY:"bg-state-failed-100 hover:bg-state-failed-200",PAUSED:"bg-state-pending-100 hover:bg-state-pending-200"}}}),c=({status:t})=>d[t],x=({status:t})=>{const a=t.split("_").map(o).join(" ");return e.jsxs(r,{className:g({status:t}),children:[e.jsx(c,{status:t}),a]})};export{x as S,c as a};
//# sourceMappingURL=index-fMpmRfhu.js.map
