import{j as e,L as n,y as i}from"./vendor-tanstack-BpuysabX.js";import{B as c,g as m,h as r,bD as o,bQ as l,C as x,q as d,dH as u,cW as p}from"./index-CREafGO4.js";import{B as j}from"./block-type-logo-BNO1-Opd.js";import{P as h}from"./python-example-snippet-DzfM6vLj.js";import{L as f}from"./lazy-markdown-rmxkK4GI.js";import"./vendor-radix-DIu41Ss_.js";import"./vendor-recharts-C2XeLr4O.js";import"./vendor-forms-Buh7U8Ay.js";import"./vendor-date-C1t37jsh.js";const g=({blockType:s})=>e.jsxs("div",{className:"flex flex-col gap-6",children:[e.jsx(B,{blockType:s}),e.jsx(N,{blockType:s})]});function B({blockType:s}){return e.jsx(c,{children:e.jsxs(m,{children:[e.jsx(r,{children:e.jsx(o,{to:"/blocks",className:"text-xl font-semibold",children:"Blocks"})}),e.jsx(l,{}),e.jsx(r,{children:e.jsx(o,{to:"/blocks/catalog",className:"text-xl font-semibold",children:"Catalog"})}),e.jsx(l,{}),e.jsx(r,{className:"text-xl font-semibold",children:s.slug})]})})}function N({blockType:s}){return e.jsx("div",{children:e.jsxs(x,{className:"p-6",children:[e.jsxs("div",{className:"flex items-center gap-4",children:[e.jsx(j,{size:"lg",logoUrl:s.logo_url}),e.jsx("h3",{className:"text-2xl font-semibold tracking-tight",children:s.name})]}),s.description&&e.jsx(b,{description:s.description,documentationUrl:s.documentation_url}),s.code_example&&e.jsxs("div",{className:"flex flex-col gap-4",children:[e.jsx("h4",{className:"text-xl font-semibold tracking-tight",children:"Example"}),e.jsx(h,{codeExample:s.code_example,name:s.name})]}),e.jsx("div",{className:"flex justify-end",children:e.jsx(d,{children:e.jsx(n,{to:"/blocks/catalog/$slug/create",params:{slug:s.slug},children:"Create"})})})]})})}function b({description:s,documentationUrl:a}){let t=s;return a&&(t=`${t} [Documentation](${a})`),e.jsx("div",{className:"prose max-w-none",children:e.jsx(f,{children:t})})}const E=function(){const{slug:a}=u.useParams(),{data:t}=i(p(a));return e.jsx(g,{blockType:t})};export{E as component};
//# sourceMappingURL=catalog_._slug-CUAvXzWA.js.map
