import{j as t}from"./vendor-tanstack-BpuysabX.js";import{s as p,af as m}from"./index-CREafGO4.js";import{R as d}from"./route-error-state-hDLBBOXq.js";import{o as u,e,a as i,s as a,n as o,d as s}from"./vendor-forms-Buh7U8Ay.js";import"./vendor-radix-DIu41Ss_.js";import"./vendor-recharts-C2XeLr4O.js";import"./vendor-date-C1t37jsh.js";u({tab:e(["runs","deployments","details"]).optional().default("runs"),"runs.page":o().int().positive().optional().default(1),"runs.limit":o().int().positive().max(100).optional().default(10),"runs.sort":e(m).optional().default("START_TIME_DESC"),"runs.flowRuns.nameLike":a().optional().default(""),"runs.flowRuns.state.name":i(a()).optional(),type:e(["span","range"]).optional(),seconds:o().int().positive().optional(),startDateTime:s().optional(),endDateTime:s().optional(),"deployments.page":o().int().positive().optional().default(1),"deployments.limit":o().int().positive().optional().default(10),"deployments.nameLike":a().optional(),"deployments.tags":i(a()).optional(),"deployments.sort":e(["CREATED_DESC","UPDATED_DESC","NAME_ASC","NAME_DESC"]).optional().default("NAME_ASC")}).optional().default({});const D=function({error:r,reset:l}){const n=p(r,"Failed to load flow");if(n.type!=="server-error"&&n.type!=="client-error")throw r;return t.jsxs("div",{className:"flex flex-col gap-4",children:[t.jsx("div",{children:t.jsx("h1",{className:"text-2xl font-semibold",children:"Flow"})}),t.jsx(d,{error:n,onRetry:l})]})};export{D as errorComponent};
//# sourceMappingURL=flow._id-BzHca_Lz.js.map
