import{d as m,S as k,j as i,g as t,h as _,c as a,o,i as n,n as f,a as P,F as h,m as s,d0 as y,d1 as g,k as v,d2 as w}from"./index-CWv_Pcr2.js";import{u as x}from"./usePageTitle-DY9DP8zZ.js";const b=m({__name:"WorkPools",setup(B){const r=k(),l={interval:3e4},e=i(r.workPools.getWorkPools,[{}],l),c=t(()=>e.response??[]),p=t(()=>e.executed&&c.value.length==0),u=t(()=>e.executed);return x("Work Pools"),(C,F)=>{const d=_("p-layout-default");return o(),a(d,{class:"work-pools"},{header:n(()=>[v(s(w))]),default:n(()=>[u.value?(o(),f(h,{key:0},[p.value?(o(),a(s(y),{key:0})):(o(),a(s(g),{key:1,onUpdate:s(e).refresh},null,8,["onUpdate"]))],64)):P("",!0)]),_:1})}}});export{b as default};
//# sourceMappingURL=WorkPools-DOS1G4Vo.js.map
