import{j as r}from"./vendor-tanstack-BpuysabX.js";import{s}from"./index-CREafGO4.js";import{R as i}from"./route-error-state-hDLBBOXq.js";import"./vendor-radix-DIu41Ss_.js";import"./vendor-recharts-C2XeLr4O.js";import"./vendor-forms-Buh7U8Ay.js";import"./vendor-date-C1t37jsh.js";const d=function({error:t,reset:e}){const o=s(t,"Failed to load artifacts");if(o.type!=="server-error"&&o.type!=="client-error")throw t;return r.jsxs("div",{className:"flex flex-col gap-4",children:[r.jsx("div",{children:r.jsx("h1",{className:"text-2xl font-semibold",children:"Artifacts"})}),r.jsx(i,{error:o,onRetry:e})]})};export{d as errorComponent};
//# sourceMappingURL=index-1kzcWE9u.js.map
