import{j as e}from"./vendor-tanstack-BpuysabX.js";import{ct as t}from"./index-CREafGO4.js";const c=({alt:o,logoUrl:s,size:r="sm"})=>e.jsx("img",{alt:o,src:s??void 0,className:a({size:r})}),a=t("border border-border bg-muted rounded",{variants:{size:{sm:"size-8",lg:"size-14"}}});export{c as B};
//# sourceMappingURL=block-type-logo-BNO1-Opd.js.map
