import{j as s}from"./vendor-tanstack-BpuysabX.js";import{B as e}from"./block-type-logo-BNO1-Opd.js";import{C as o}from"./index-CREafGO4.js";import{L as t}from"./lazy-markdown-rmxkK4GI.js";function n({blockType:r}){return s.jsxs(o,{className:"p-6 self-start",children:[s.jsxs("div",{className:"flex items-center gap-4",children:[s.jsx(e,{size:"lg",logoUrl:r.logo_url}),s.jsx("h4",{className:"text-xl font-semibold tracking-tight",children:r.name})]}),r.description&&s.jsx("div",{className:"prose max-w-none overflow-y-auto text-sm",children:s.jsx(t,{children:r.description})})]})}export{n as B};
//# sourceMappingURL=block-type-details-CJxHA2oB.js.map
