import{j as e}from"./vendor-tanstack-BpuysabX.js";import{B as s,g as r,h as t}from"./index-CREafGO4.js";const c=()=>e.jsx("div",{className:"flex items-center gap-2",children:e.jsx(s,{children:e.jsx(r,{children:e.jsx(t,{className:"text-xl font-semibold",children:"Deployments"})})})});export{c as D};
//# sourceMappingURL=header-CcU71fLt.js.map
