import{j as r}from"./vendor-tanstack-BpuysabX.js";import{s as i}from"./index-CREafGO4.js";import{R as n}from"./route-error-state-hDLBBOXq.js";import"./vendor-radix-DIu41Ss_.js";import"./vendor-recharts-C2XeLr4O.js";import"./vendor-forms-Buh7U8Ay.js";import"./vendor-date-C1t37jsh.js";const f=function({error:t,reset:e}){const o=i(t,"Failed to load automation");if(o.type!=="server-error"&&o.type!=="client-error")throw t;return r.jsxs("div",{className:"flex flex-col gap-4",children:[r.jsx("div",{children:r.jsx("h1",{className:"text-2xl font-semibold",children:"Edit Automation"})}),r.jsx(n,{error:o,onRetry:e})]})};export{f as errorComponent};
//# sourceMappingURL=automation_._id.edit-yaxZ2ejc.js.map
