import{r as o,j as r}from"./vendor-tanstack-BpuysabX.js";import{ao as i,ap as t,aq as l}from"./vendor-radix-DIu41Ss_.js";import{b4 as c,cj as n}from"./index-CREafGO4.js";const d=o.forwardRef(({className:s,...a},e)=>r.jsx(i,{className:c("grid gap-2",s),...a,ref:e}));d.displayName=i.displayName;const m=o.forwardRef(({className:s,...a},e)=>r.jsx(t,{ref:e,className:c("aspect-square h-4 w-4 rounded-full border border-primary text-primary ring-offset-background focus:outline-none focus-visible:ring-2 focus-visible:ring-ring focus-visible:ring-offset-2 disabled:cursor-not-allowed disabled:opacity-50",s),...a,children:r.jsx(l,{className:"flex items-center justify-center",children:r.jsx(n,{className:"h-2.5 w-2.5 fill-current text-current"})})}));m.displayName=t.displayName;export{d as R,m as a};
//# sourceMappingURL=radio-group-B1MsDBBw.js.map
