import{d as r,h as s,c as n,o as l,i as e,k as o,m as a,d3 as u,d4 as _}from"./index-CWv_Pcr2.js";import{u as c}from"./usePageTitle-DY9DP8zZ.js";const k=r({__name:"WorkPoolCreate",setup(p){return c("Create Work Pool"),(d,f)=>{const t=s("p-layout-default");return l(),n(t,null,{header:e(()=>[o(a(_))]),default:e(()=>[o(a(u))]),_:1})}}});export{k as default};
//# sourceMappingURL=WorkPoolCreate-BPwYmO1e.js.map
