import{j as r}from"./vendor-tanstack-BpuysabX.js";import{s}from"./index-CREafGO4.js";import{R as i}from"./route-error-state-hDLBBOXq.js";import{o as n,e as l}from"./vendor-forms-Buh7U8Ay.js";import"./vendor-radix-DIu41Ss_.js";import"./vendor-recharts-C2XeLr4O.js";import"./vendor-date-C1t37jsh.js";n({tab:l(["Details","Runs","Work Queues","Workers","Deployments"]).default("Runs")});const j=function({error:e,reset:t}){const o=s(e,"Failed to load work pool");if(o.type!=="server-error"&&o.type!=="client-error")throw e;return r.jsxs("div",{className:"flex flex-col gap-4",children:[r.jsx("div",{children:r.jsx("h1",{className:"text-2xl font-semibold",children:"Work Pool"})}),r.jsx(i,{error:o,onRetry:t})]})};export{j as errorComponent};
//# sourceMappingURL=work-pool._workPoolName-CNUeKSev.js.map
