import{j as s}from"./vendor-tanstack-BpuysabX.js";import{q as n,I as r,y as x}from"./index-CREafGO4.js";function m({label:e,value:t,copyable:o,copyValue:i}){const a=()=>{const c=i??(typeof t=="string"?t:"");navigator.clipboard.writeText(c),x.success(`${e} copied`)};return s.jsxs("dl",{className:"flex flex-col gap-1",children:[s.jsx("dt",{className:"text-muted-foreground text-sm",children:e}),s.jsxs("dd",{className:"text-sm flex items-center gap-2",children:[t,o&&s.jsx(n,{variant:"ghost",size:"icon",onClick:a,children:s.jsx(r,{id:"Copy",className:"size-4"})})]})]})}export{m as K};
//# sourceMappingURL=key-value-Dw-F7Q47.js.map
