import{j as e}from"./vendor-tanstack-BpuysabX.js";import{ar as d,as as r,at as i,au as u,av as p,aw as m,ax as c,ay as l}from"./vendor-radix-DIu41Ss_.js";import{b4 as n}from"./index-CREafGO4.js";function g({...t}){return e.jsx(d,{"data-slot":"dropdown-menu",...t})}function w({...t}){return e.jsx(r,{"data-slot":"dropdown-menu-trigger",...t})}function b({className:t,sideOffset:a=4,...o}){return e.jsx(i,{children:e.jsx(u,{"data-slot":"dropdown-menu-content",sideOffset:a,className:n("bg-popover text-popover-foreground data-[state=open]:animate-in data-[state=closed]:animate-out data-[state=closed]:fade-out-0 data-[state=open]:fade-in-0 data-[state=closed]:zoom-out-95 data-[state=open]:zoom-in-95 data-[side=bottom]:slide-in-from-top-2 data-[side=left]:slide-in-from-right-2 data-[side=right]:slide-in-from-left-2 data-[side=top]:slide-in-from-bottom-2 z-50 min-w-[8rem] overflow-hidden rounded-md border p-1 shadow-md",t),...o})})}function j({...t}){return e.jsx(l,{"data-slot":"dropdown-menu-group",...t})}function h({className:t,inset:a,variant:o="default",...s}){return e.jsx(m,{"data-slot":"dropdown-menu-item","data-inset":a,"data-variant":o,className:n("focus:bg-accent focus:text-accent-foreground data-[variant=destructive]:text-destructive data-[variant=destructive]:focus:bg-destructive/10 dark:data-[variant=destructive]:focus:bg-destructive/20 data-[variant=destructive]:focus:text-destructive data-[variant=destructive]:*:[svg]:!text-destructive [&_svg:not([class*='text-'])]:text-muted-foreground relative flex cursor-default items-center gap-2 rounded-sm px-2 py-1.5 text-sm outline-hidden select-none data-[disabled]:pointer-events-none data-[disabled]:opacity-50 data-[inset]:pl-8 [&_svg]:pointer-events-none [&_svg]:shrink-0 [&_svg:not([class*='size-'])]:size-4",t),...s})}function D({className:t,inset:a,...o}){return e.jsx(p,{"data-slot":"dropdown-menu-label","data-inset":a,className:n("px-2 py-1.5 text-sm font-medium data-[inset]:pl-8",t),...o})}function M({className:t,...a}){return e.jsx(c,{"data-slot":"dropdown-menu-separator",className:n("bg-border -mx-1 my-1 h-px",t),...a})}export{g as D,w as a,b,D as c,h as d,M as e,j as f};
//# sourceMappingURL=dropdown-menu-DZExFL3b.js.map
