import{d,S as i,j as y,g as s,h as _,c as a,o as t,i as n,n as f,a as k,F as v,m as o,bX as C,bU as b,k as h,bY as x}from"./index-CWv_Pcr2.js";import{u as D}from"./usePageTitle-DY9DP8zZ.js";const N=d({__name:"Deployments",setup(g){const l=i(),c={interval:3e4},e=y(l.deployments.getDeploymentsCount,[{}],c),p=s(()=>e.response??0),r=s(()=>e.executed&&p.value===0),u=s(()=>e.executed);return D("Deployments"),(B,S)=>{const m=_("p-layout-default");return t(),a(m,{class:"deployments"},{header:n(()=>[h(o(x))]),default:n(()=>[u.value?(t(),f(v,{key:0},[r.value?(t(),a(o(C),{key:0})):(t(),a(o(b),{key:1,onDelete:o(e).refresh},null,8,["onDelete"]))],64)):k("",!0)]),_:1})}}});export{N as default};
//# sourceMappingURL=Deployments-Deemmdue.js.map
