import{d as c,S as _,v as u,I as k,h as m,c as d,o as i,i as l,k as r,m as e,d5 as w,d6 as f}from"./index-CWv_Pcr2.js";import{u as P}from"./usePageTitle-DY9DP8zZ.js";const N=c({__name:"WorkPoolEdit",async setup(h){let o,a;const s=_(),n=u("workPoolName"),t=([o,a]=k(()=>s.workPools.getWorkPoolByName(n.value)),o=await o,a(),o);return P("Create Work Pool"),(v,y)=>{const p=m("p-layout-default");return i(),d(p,null,{header:l(()=>[r(e(f),{"work-pool":e(t)},null,8,["work-pool"])]),default:l(()=>[r(e(w),{"work-pool":e(t)},null,8,["work-pool"])]),_:1})}}});export{N as default};
//# sourceMappingURL=WorkPoolEdit-yleXRD87.js.map
