import{d as I,S as N,v as h,j,g as p,ae as D,al as R,G as d,aH as T,h as c,c as n,o as l,i as e,n as V,a as i,k as s,m as t,aI as $,aJ as H,aK as _,B as L,t as z,aL as v,a6 as E,aM as m,aN as G}from"./index-CWv_Pcr2.js";import{u as J}from"./usePageTitle-DY9DP8zZ.js";const K={key:0},F=I({__name:"Artifact",setup(M){const y=N(),b=h("artifactId"),k=j(y.artifacts.getArtifact,[b]),a=p(()=>k.response),r=D(!1),w=[{label:"Artifact"},{label:"Details"},{label:"Raw"}],u=R("tab","Artifact"),g=p(()=>a.value?`${d.info.artifact}: ${a.value.key??T(a.value.type)}`:d.info.artifact);return J(g),(P,o)=>{const x=c("p-divider"),A=c("p-button"),C=c("p-content"),S=c("p-tabs"),B=c("p-layout-well");return l(),n(B,{class:"artifact"},{header:e(()=>[a.value?(l(),n(t(G),{key:0,artifact:a.value},null,8,["artifact"])):i("",!0)]),well:e(()=>[a.value?(l(),n(t(m),{key:0,artifact:a.value,alternate:""},null,8,["artifact"])):i("",!0)]),default:e(()=>[a.value?(l(),V("section",K,[s(t($),{artifact:a.value},null,8,["artifact"]),s(x),t(H).xl?(l(),n(C,{key:0},{default:e(()=>[s(t(_),{artifact:a.value},null,8,["artifact"]),s(A,{class:"artifact__raw-data-button",small:"",onClick:o[0]||(o[0]=f=>r.value=!r.value)},{default:e(()=>[L(z(r.value?"Hide":"Show")+" raw data ",1)]),_:1}),r.value?(l(),n(t(v),{key:0,artifact:a.value},null,8,["artifact"])):i("",!0)]),_:1})):(l(),n(S,{key:1,selected:t(u),"onUpdate:selected":o[1]||(o[1]=f=>E(u)?u.value=f:null),tabs:w},{artifact:e(()=>[s(t(_),{artifact:a.value},null,8,["artifact"])]),details:e(()=>[s(t(m),{artifact:a.value},null,8,["artifact"])]),raw:e(()=>[s(t(v),{artifact:a.value},null,8,["artifact"])]),_:1},8,["selected"]))])):i("",!0)]),_:1})}}});export{F as default};
//# sourceMappingURL=Artifact-Fl71KMc1.js.map
