import{d as v,g as y,n as u,o as i,k as e,i as t,F as x,x as C,t as f,m as n,dw as w,I as B,h as l,c as P,dx as V,a6 as A,dy as p,z as F}from"./index-CWv_Pcr2.js";import{u as M}from"./usePageTitle-DY9DP8zZ.js";import{u as T}from"./usePrefectApi-DefSgQ13.js";import"./mapper-CBOLB2hm.js";const j={class:"settings-block"},z=v({__name:"SettingsCodeBlock",props:{engineSettings:{}},setup(d){const s=d,o=y(()=>Object.entries(s.engineSettings));return(g,c)=>(i(),u("div",j,[e(n(w),{multiline:""},{default:t(()=>[(i(!0),u(x,null,C(o.value,(a,r)=>(i(),u("div",{key:r,class:"settings-block--code-line"},f(a[0])+": "+f(a[1]),1))),128))]),_:1})]))}}),O=v({__name:"Settings",async setup(d){let s,o;const g=[{text:"Settings"}],c=T(),[a,r]=([s,o]=B(()=>Promise.all([c.admin.getSettings(),c.admin.getVersion()])),s=await s,o(),s);return M("Settings"),(D,m)=>{const S=l("p-key-value"),b=l("p-theme-toggle"),_=l("p-label"),k=l("p-layout-default");return i(),P(k,{class:"settings"},{header:t(()=>[e(n(F),{crumbs:g},{actions:t(()=>[e(S,{class:"settings__version",label:"Version",value:n(r),alternate:""},null,8,["value"])]),_:1})]),default:t(()=>[e(_,{label:"Theme"},{default:t(()=>[e(b)]),_:1}),e(_,{label:"Color Mode",class:"settings__color-mode"},{default:t(()=>[e(n(V),{selected:n(p),"onUpdate:selected":m[0]||(m[0]=h=>A(p)?p.value=h:null)},null,8,["selected"])]),_:1}),e(_,{label:"Server Settings"},{default:t(()=>[e(z,{class:"settings__code-block","engine-settings":n(a)},null,8,["engine-settings"])]),_:1})]),_:1})}}});export{O as default};
//# sourceMappingURL=Settings-D_Gxv9yh.js.map
