import{j as o}from"./vendor-tanstack-BpuysabX.js";import{s}from"./index-CREafGO4.js";import{R as a}from"./route-error-state-hDLBBOXq.js";import{V as i}from"./header-ekl8Ab7Q.js";import"./vendor-radix-DIu41Ss_.js";import"./vendor-recharts-C2XeLr4O.js";import"./vendor-forms-Buh7U8Ay.js";import"./vendor-date-C1t37jsh.js";const j=function({error:e,reset:t}){const r=s(e,"Failed to load variables");if(r.type!=="server-error"&&r.type!=="client-error")throw e;return o.jsxs("div",{className:"flex flex-col gap-4",children:[o.jsx(i,{}),o.jsx(a,{error:r,onRetry:t})]})};export{j as errorComponent};
//# sourceMappingURL=index-DRCdBZDl.js.map
