import{j as r}from"./vendor-tanstack-BpuysabX.js";import{s as n}from"./index-CREafGO4.js";import{R as s}from"./route-error-state-hDLBBOXq.js";import"./vendor-radix-DIu41Ss_.js";import"./vendor-recharts-C2XeLr4O.js";import"./vendor-forms-Buh7U8Ay.js";import"./vendor-date-C1t37jsh.js";const f=function({error:o,reset:t}){const e=n(o,"Failed to load flow run");return r.jsxs("div",{className:"flex flex-col gap-4",children:[r.jsx("div",{children:r.jsx("h1",{className:"text-2xl font-semibold",children:"Flow Run"})}),r.jsx(s,{error:e,onRetry:t})]})};export{f as errorComponent};
//# sourceMappingURL=flow-run._id-Bj9-oCQg.js.map
