import{j as t}from"./vendor-tanstack-BpuysabX.js";import{s as i,ad as l,ae as p,af as u}from"./index-CREafGO4.js";import{R as f}from"./route-error-state-hDLBBOXq.js";import{o as d,s as o,e as a,n as e,b as m}from"./vendor-forms-Buh7U8Ay.js";import"./vendor-radix-DIu41Ss_.js";import"./vendor-recharts-C2XeLr4O.js";import"./vendor-date-C1t37jsh.js";d({tab:a(["flow-runs","task-runs"]).optional().default("flow-runs"),page:e().int().positive().optional().default(1).catch(1),limit:e().int().positive().optional().catch(void 0),sort:a(u).optional().default("START_TIME_DESC"),"hide-subflows":m().optional().default(!1),"flow-run-search":o().optional().default(""),state:o().optional().default(""),flows:o().optional().default(""),deployments:o().optional().default(""),"work-pools":o().optional().default(""),tags:o().optional().default(""),range:a(p).optional(),start:o().optional(),end:o().optional(),"task-runs-page":e().int().positive().optional().default(1).catch(1),"task-runs-limit":e().int().positive().optional().catch(void 0),"task-runs-sort":a(l).optional().default("EXPECTED_START_TIME_DESC"),"task-run-search":o().optional().default("")});const v=function({error:s,reset:n}){const r=i(s,"Failed to load runs");if(r.type!=="server-error"&&r.type!=="client-error")throw s;return t.jsxs("div",{className:"flex flex-col gap-4",children:[t.jsx("div",{children:t.jsx("h1",{className:"text-2xl font-semibold",children:"Runs"})}),t.jsx(f,{error:r,onRetry:n})]})};export{v as errorComponent};
//# sourceMappingURL=index-5TYdnvYr.js.map
