import{d as k,S as c,v as l,I as w,h as _,c as d,o as i,i as r,k as n,m as o,dc as P,dd as f}from"./index-CWv_Pcr2.js";import{u as Q}from"./usePageTitle-DY9DP8zZ.js";const x=k({__name:"WorkPoolQueueEdit",async setup(N){let e,u;const s=c(),a=l("workPoolName"),t=l("workPoolQueueName"),m=([e,u]=w(()=>s.workPoolQueues.getWorkPoolQueueByName(a.value,t.value)),e=await e,u(),e);return Q("Edit Work Pool Queue"),(h,v)=>{const p=_("p-layout-default");return i(),d(p,null,{header:r(()=>[n(o(f),{"work-pool-name":o(a),"work-pool-queue-name":o(t)},null,8,["work-pool-name","work-pool-queue-name"])]),default:r(()=>[n(o(P),{"work-pool-name":o(a),"work-pool-queue":o(m)},null,8,["work-pool-name","work-pool-queue"])]),_:1})}}});export{x as default};
//# sourceMappingURL=WorkPoolQueueEdit-W3ORMNck.js.map
