const __vite__mapDeps=(i,m=__vite__mapDeps,d=(m.f||(m.f=["assets/python-input-CziWCQkr.js","assets/vendor-tanstack-BpuysabX.js","assets/vendor-codemirror-q5gL9Jwo.js","assets/index-CREafGO4.js","assets/vendor-radix-DIu41Ss_.js","assets/vendor-recharts-C2XeLr4O.js","assets/vendor-forms-Buh7U8Ay.js","assets/vendor-date-C1t37jsh.js","assets/index-CPPsTKEF.css"])))=>i.map(i=>d[i]);
import{j as e,r as s}from"./vendor-tanstack-BpuysabX.js";import{bd as p,cT as i}from"./index-CREafGO4.js";const a=s.lazy(()=>i(()=>import("./python-input-CziWCQkr.js"),__vite__mapDeps([0,1,2,3,4,5,6,7,8])).then(t=>({default:t.PythonInput})));function c({className:t,...n}){return e.jsx(s.Suspense,{fallback:e.jsx(p,{className:`min-h-[200px] ${t??""}`}),children:e.jsx(a,{className:t,...n})})}function h({codeExample:t,name:n}){const o=s.useMemo(()=>{const[,r=""]=t.match(/```python([\S\s]*?)```/)??[];return r.replace("BLOCK_NAME",n??"block-name").trim()},[t,n]);return e.jsx("div",{className:"flex flex-col gap-4",children:e.jsx(c,{className:"p-2",value:o,disabled:!0,copy:!0,hideLineNumbers:!0})})}export{h as P};
//# sourceMappingURL=python-example-snippet-DzfM6vLj.js.map
