import{r as n}from"./vendor-tanstack-BpuysabX.js";const r="Prefect Server",i=" • ";function l(...o){const t=[...o.filter(e=>e!=null),r].join(i);n.useEffect(()=>(document.title=t,()=>{document.title=r}),[t])}export{l as u};
//# sourceMappingURL=use-page-title-B8bZfuyN.js.map
