import{j as e}from"./vendor-tanstack-BpuysabX.js";import{S as C,a as S,b as D,ax as a,c as A,e as L}from"./index-CREafGO4.js";const E={COMPLETED:"Completed",RUNNING:"Running",SCHEDULED:"Scheduled",PENDING:"Pending",FAILED:"Failed",CANCELLED:"Cancelled",CANCELLING:"Cancelling",CRASHED:"Crashed",PAUSED:"Paused"},N=["Scheduled","Late","Resuming","AwaitingRetry","AwaitingConcurrencySlot","Pending","Paused","Suspended","Running","Retrying","Completed","Cached","Cancelled","Cancelling","Crashed","Failed","TimedOut"],g={Scheduled:"SCHEDULED",Late:"SCHEDULED",Resuming:"SCHEDULED",AwaitingRetry:"SCHEDULED",AwaitingConcurrencySlot:"SCHEDULED",Pending:"PENDING",Paused:"PAUSED",Suspended:"PAUSED",Running:"RUNNING",Retrying:"RUNNING",Completed:"COMPLETED",Cached:"COMPLETED",Cancelled:"CANCELLED",Cancelling:"CANCELLING",Crashed:"CRASHED",Failed:"FAILED",TimedOut:"FAILED"},R=N.filter(t=>t!=="Scheduled"),u=["COMPLETED","FAILED","CANCELLED","CRASHED"],P=({value:t,onValueChange:l,placeholder:i="Select state",terminalOnly:c=!1,excludeState:s})=>{const d=Object.entries(E).filter(([n])=>!(c&&!u.includes(n)||s&&n===s));return e.jsxs(C,{value:t,onValueChange:l,children:[e.jsx(S,{"aria-label":"select state",className:"w-full",children:e.jsx(D,{placeholder:i,children:t&&e.jsx(a,{type:t,name:E[t]})})}),e.jsx(A,{className:"w-full min-w-[300px]",children:d.map(([n,r])=>e.jsx(L,{value:n,className:"flex items-center",children:e.jsx(a,{type:n,name:r})},n))})]})};export{E as R,P as S,N as a,g as b,R as c};
//# sourceMappingURL=index-DVVSN4AL.js.map
