import{d as f,S as i,j as _,g as o,h as w,c as s,o as t,i as n,n as k,a as C,F as b,m as a,bO as h,bP as y,k as v,bQ as x}from"./index-CWv_Pcr2.js";import{u as F}from"./usePageTitle-DY9DP8zZ.js";const I=f({__name:"Flows",setup(g){const l=i(),c={interval:3e4},e=_(l.flows.getFlowsCount,[{}],c),r=o(()=>e.response??0),u=o(()=>e.executed&&r.value===0),p=o(()=>e.executed),m=()=>{e.refresh()};return F("Flows"),(B,S)=>{const d=w("p-layout-default");return t(),s(d,{class:"flows"},{header:n(()=>[v(a(x))]),default:n(()=>[p.value?(t(),k(b,{key:0},[u.value?(t(),s(a(h),{key:0})):(t(),s(a(y),{key:1,selectable:"",onDelete:m}))],64)):C("",!0)]),_:1})}}});export{I as default};
//# sourceMappingURL=Flows-P8UhqLj2.js.map
