import{d as e,c as o,o as a,m as t,dz as r}from"./index-CWv_Pcr2.js";import{u as s}from"./usePageTitle-DY9DP8zZ.js";const _=e({__name:"404",setup(n){return s("Page Not Found"),(c,m)=>(a(),o(t(r)))}});export{_ as default};
//# sourceMappingURL=404-DMzJEQmf.js.map
