import{d as b,S as m,ae as y,g as o,ck as f,j as _,h as d,c as g,o as v,i as s,k as t,m as l,cl as C,cm as B}from"./index-CWv_Pcr2.js";import{u as T}from"./usePageTitle-DY9DP8zZ.js";const j=b({__name:"BlocksCatalog",setup(S){const c=m(),a=y(null),p=o(()=>a.value?[a.value]:[]),{filter:n}=f({blockSchemas:{blockCapabilities:p}}),i=_(c.blockTypes.getBlockTypes,[n]),u=o(()=>i.response??[]);return T("Blocks Catalog"),(h,e)=>{const r=d("p-layout-default");return v(),g(r,{class:"blocks-catalog"},{header:s(()=>[t(l(B))]),default:s(()=>[t(l(C),{capability:a.value,"onUpdate:capability":e[0]||(e[0]=k=>a.value=k),"block-types":u.value},null,8,["capability","block-types"])]),_:1})}}});export{j as default};
//# sourceMappingURL=BlocksCatalog-C2ToVYfn.js.map
