import{d as m,S as y,v as _,j as k,g as s,G as o,h as v,c as e,o as t,i as n,a as r,n as d,m as c,aI as g,F as h,aO as C,aP as x}from"./index-CWv_Pcr2.js";import{u as B}from"./usePageTitle-DY9DP8zZ.js";const P=m({__name:"ArtifactKey",setup(F){const l=y(),u=_("artifactKey"),i=k(l.artifacts.getArtifactCollection,[u]),a=s(()=>i.response),f=s(()=>a.value?`${o.info.artifact}: ${a.value.key}`:o.info.artifact);return B(f),(K,S)=>{const p=v("p-layout-default");return t(),e(p,{class:"artifact"},{header:n(()=>[a.value?(t(),e(c(x),{key:0,artifact:a.value},null,8,["artifact"])):r("",!0)]),default:n(()=>[a.value?(t(),e(c(g),{key:0,artifact:a.value},null,8,["artifact"])):r("",!0),a.value?(t(),d(h,{key:1},[a.value.key?(t(),e(c(C),{key:0,"artifact-key":a.value.key},null,8,["artifact-key"])):r("",!0)],64)):r("",!0)]),_:1})}}});export{P as default};
//# sourceMappingURL=ArtifactKey-BXZh5bFV.js.map
