import{j as t,L as o}from"./vendor-tanstack-BpuysabX.js";import{E as n,t as e,v as s,w as r,x as i,q as a}from"./index-CREafGO4.js";import"./vendor-radix-DIu41Ss_.js";import"./vendor-recharts-C2XeLr4O.js";import"./vendor-forms-Buh7U8Ay.js";import"./vendor-date-C1t37jsh.js";function p(){return t.jsxs(n,{children:[t.jsx(e,{id:"Search"}),t.jsx(s,{children:"Flow Run Not Found"}),t.jsx(r,{children:"The flow run you are looking for does not exist or has been deleted."}),t.jsx(i,{children:t.jsx(a,{asChild:!0,children:t.jsx(o,{to:"/runs",children:"Back to Runs"})})})]})}const h=p;export{h as notFoundComponent};
//# sourceMappingURL=flow-run._id-DrdQ7JbY.js.map
