import{d as m,S as p,j as d,g as s,h as _,c as a,o as t,i as c,n as k,a as i,F as f,m as o,ch as h,ci as y,k as B,cj as C}from"./index-CWv_Pcr2.js";import{u as x}from"./usePageTitle-DY9DP8zZ.js";const E=m({__name:"Blocks",setup(g){const n=p(),e=d(n.blockDocuments.getBlockDocumentsCount),l=s(()=>e.executed&&e.response==0),r=s(()=>e.executed);return x("Blocks"),(D,b)=>{const u=_("p-layout-default");return t(),a(u,{class:"blocks"},{header:c(()=>[B(o(C))]),default:c(()=>[r.value?(t(),k(f,{key:0},[l.value?(t(),a(o(h),{key:0})):(t(),a(o(y),{key:1,onDelete:o(e).refresh},null,8,["onDelete"]))],64)):i("",!0)]),_:1})}}});export{E as default};
//# sourceMappingURL=Blocks-Cyyosqjt.js.map
