import{d as S,S as h,v as w,u as x,g as r,aJ as D,al as j,bv as R,j as T,h as l,c,o as n,i as a,k as z,m as t,a6 as A,a as o,cy as B,cz as u,cA as I,bg as N}from"./index-CWv_Pcr2.js";import{u as V}from"./usePageTitle-DY9DP8zZ.js";const $=S({__name:"ConcurrencyLimit",setup(J){const m=h(),y=w("concurrencyLimitId"),p=x(),v=r(()=>[{label:"Details",hidden:D.xl},{label:"Active Task Runs"}]),s=j("tab","Details"),{tabs:d}=R(v,s),_={interval:3e5},b=T(m.concurrencyLimits.getConcurrencyLimit,[y.value],_),e=r(()=>b.response);function f(){p.push(N.concurrencyLimits())}const L=r(()=>e.value?`Concurrency Limit: ${e.value.tag}`:"Concurrency Limit");return V(L),(O,i)=>{const k=l("p-tabs"),C=l("p-layout-well");return n(),c(C,{class:"concurrencyLimit"},{header:a(()=>[e.value?(n(),c(t(I),{key:0,"concurrency-limit":e.value,onDelete:f},null,8,["concurrency-limit"])):o("",!0)]),well:a(()=>[e.value?(n(),c(t(u),{key:0,alternate:"","concurrency-limit":e.value},null,8,["concurrency-limit"])):o("",!0)]),default:a(()=>[z(k,{selected:t(s),"onUpdate:selected":i[0]||(i[0]=g=>A(s)?s.value=g:null),tabs:t(d)},{details:a(()=>[e.value?(n(),c(t(u),{key:0,"concurrency-limit":e.value},null,8,["concurrency-limit"])):o("",!0)]),"active-task-runs":a(()=>[e.value?.activeSlots?(n(),c(t(B),{key:0,"active-slots":e.value.activeSlots},null,8,["active-slots"])):o("",!0)]),_:1},8,["selected","tabs"])]),_:1})}}});export{$ as default};
//# sourceMappingURL=ConcurrencyLimit-BNxOQ0wP.js.map
