import{d as f,S as x,f as b,v as c,g as r,aD as h,aE as k,I as y,h as C,c as E,a as F,m as e,o as D,i as n,k as s,aF as I,z as g,aG as w}from"./index-CWv_Pcr2.js";import{u as L}from"./usePageTitle-DY9DP8zZ.js";const j=f({__name:"Event",async setup(B){let a,o;const v=x(),u=b(),l=c("eventDate"),m=c("eventId"),_=r(()=>h(l.value)),d=k({startDate:_,eventId:[m.value]}),t=([a,o]=y(()=>v.events.getFirstEvent(d.value)),a=await a,o(),a),p=r(()=>[{text:"Event Feed",to:u.events()},{text:t.eventLabel}]);return L(`Event: ${t.eventLabel}`),(N,S)=>{const i=C("p-layout-default");return e(t)?(D(),E(i,{key:0,class:"event"},{header:n(()=>[s(e(g),{crumbs:p.value},{actions:n(()=>[s(e(w),{event:e(t)},null,8,["event"])]),_:1},8,["crumbs"])]),default:n(()=>[s(e(I),{event:e(t)},null,8,["event"])]),_:1})):F("",!0)}}});export{j as default};
//# sourceMappingURL=Event-3EaU9ZB-.js.map
