import{j as o}from"./vendor-tanstack-BpuysabX.js";import{s}from"./index-CREafGO4.js";import{R as n}from"./route-error-state-hDLBBOXq.js";import{o as p,e as l,a as m,s as e,n as a}from"./vendor-forms-Buh7U8Ay.js";import"./vendor-radix-DIu41Ss_.js";import"./vendor-recharts-C2XeLr4O.js";import"./vendor-date-C1t37jsh.js";p({name:e().optional(),page:a().int().positive().optional().default(1).catch(1),limit:a().int().positive().max(100).optional().default(10).catch(10),tags:m(e()).optional(),sort:l(["CREATED_DESC","UPDATED_DESC","NAME_ASC","NAME_DESC"]).optional().default("NAME_ASC")}).optional().default({});const h=function({error:t,reset:i}){const r=s(t,"Failed to load flows");if(r.type!=="server-error"&&r.type!=="client-error")throw t;return o.jsxs("div",{className:"flex flex-col gap-4",children:[o.jsx("div",{children:o.jsx("h1",{className:"text-2xl font-semibold",children:"Flows"})}),o.jsx(n,{error:r,onRetry:i})]})};export{h as errorComponent};
//# sourceMappingURL=index-CZA72wOw.js.map
