import{y as r,j as t}from"./vendor-tanstack-BpuysabX.js";import{cE as m,B as l,g as c,h as s,bD as u,bQ as d,bS as x,dJ as p}from"./index-CREafGO4.js";import{D as j}from"./delete-confirmation-dialog-DxI68qN2.js";import{u as f,A as b,a as g,b as h,c as A,d as D}from"./use-delete-automation-confirmation-dialog-C5uIRRA8.js";import"./vendor-radix-DIu41Ss_.js";import"./vendor-recharts-C2XeLr4O.js";import"./vendor-forms-Buh7U8Ay.js";import"./vendor-date-C1t37jsh.js";import"./work-queue-icon-text-8VfHurcv.js";import"./index-fMpmRfhu.js";import"./dropdown-menu-DZExFL3b.js";import"./use-delete-confirmation-dialog-BgQibraR.js";const N=({id:o})=>{const[a,i]=f(),{data:e}=r(m(o)),n=()=>i(e,{shouldNavigate:!0});return t.jsxs(t.Fragment,{children:[t.jsxs("div",{className:"flex flex-col gap-4",children:[t.jsxs("header",{className:"flex items-center justify-between",children:[t.jsx(B,{automation:e}),t.jsxs("div",{className:"flex items-center gap-2",children:[t.jsx(b,{automation:e}),t.jsx(g,{id:e.id,onDelete:n})]})]}),t.jsxs("div",{className:"flex flex-col gap-4",children:[t.jsx(h,{automation:e}),t.jsx(A,{automation:e}),t.jsx(D,{automation:e})]})]}),t.jsx(j,{...a})]})},B=({automation:o})=>t.jsx(l,{children:t.jsxs(c,{children:[t.jsx(s,{children:t.jsx(u,{to:"/automations",className:"text-xl font-semibold",children:"Automations"})}),t.jsx(d,{}),t.jsx(s,{className:"text-xl font-semibold",children:t.jsx(x,{children:o.name})})]})}),F=function(){const{id:a}=p.useParams();return t.jsx(N,{id:a})};export{F as component};
//# sourceMappingURL=automation._id-CXWlBCPo.js.map
