import{j as o}from"./vendor-tanstack-BpuysabX.js";import{s}from"./index-CREafGO4.js";import{A as i}from"./automations-header-48H3dmAW.js";import{R as m}from"./route-error-state-hDLBBOXq.js";import"./vendor-radix-DIu41Ss_.js";import"./vendor-recharts-C2XeLr4O.js";import"./vendor-forms-Buh7U8Ay.js";import"./vendor-date-C1t37jsh.js";const d=function({error:t,reset:e}){const r=s(t,"Failed to load automations");if(r.type!=="server-error"&&r.type!=="client-error")throw t;return o.jsxs("div",{className:"flex flex-col gap-4",children:[o.jsx(i,{}),o.jsx(m,{error:r,onRetry:e})]})};export{d as errorComponent};
//# sourceMappingURL=index-zx70W7rJ.js.map
