import{d as k,S as i,v as y,g as o,bH as b,h as m,c as _,a as f,o as v,i as l,k as t,m as c,cn as T,co as d}from"./index-CWv_Pcr2.js";import{u as g}from"./usePageTitle-DY9DP8zZ.js";const x=k({__name:"BlocksCatalogView",setup(S){const s=i(),a=y("blockTypeSlug"),n=o(()=>a.value?[a.value]:null),u=b(s.blockTypes.getBlockTypeBySlug,n),e=o(()=>u.response),p=o(()=>e.value?`Block Type: ${e.value.name}`:null);return g(p),(B,C)=>{const r=m("p-layout-default");return e.value?(v(),_(r,{key:0,class:"blocks-catalog-view"},{header:l(()=>[t(c(d),{"block-type":e.value},null,8,["block-type"])]),default:l(()=>[t(c(T),{"block-type":e.value},null,8,["block-type"])]),_:1})):f("",!0)}}});export{x as default};
//# sourceMappingURL=BlocksCatalogView-BkK-Yq5v.js.map
