import{r as n}from"./vendor-tanstack-BpuysabX.js";const m=()=>{const[s,e]=n.useState({isOpen:!1,title:"",description:"",onConfirm:()=>{},onClose:()=>e(o=>({...o,isOpen:!1}))}),i=n.useCallback(({title:o="Confirm Deletion",description:a="Are you sure you want to delete this item? This action cannot be undone.",confirmText:l,isLoading:r=!1,onConfirm:c})=>{const t=()=>e(f=>({...f,isOpen:!1}));e({isOpen:!0,title:o,description:a,confirmText:l,isLoading:r,onConfirm:()=>{c(),t()},onClose:t})},[]);return[s,i]};export{m as u};
//# sourceMappingURL=use-delete-confirmation-dialog-BgQibraR.js.map
