import{u as l,r as d,j as s}from"./vendor-tanstack-BpuysabX.js";import{bC as c,bA as p,co as m,aj as f,T as b,cB as y}from"./index-CREafGO4.js";import{az as k,aA as w,aB as Q}from"./vendor-radix-DIu41Ss_.js";const j=()=>{const[{data:o},{data:r},{data:a},{data:u},{data:n},{data:i}]=l({queries:[c(),p(),m(),f(),b(),y()]});return{resourceOptions:d.useMemo(()=>{const t=[];for(const e of o??[])t.push({id:e.id,name:e.name,type:"automation",resourceId:`prefect.automation.${e.id}`});for(const e of r??[])t.push({id:e.id,name:e.name??"",type:"block",resourceId:`prefect.block-document.${e.id}`});for(const e of a??[])t.push({id:e.id,name:e.name,type:"deployment",resourceId:`prefect.deployment.${e.id}`});for(const e of u??[])t.push({id:e.id,name:e.name,type:"flow",resourceId:`prefect.flow.${e.id}`});for(const e of n??[])t.push({id:e.name,name:e.name,type:"work-pool",resourceId:`prefect.work-pool.${e.name}`});for(const e of i??[])t.push({id:e.id,name:e.name,type:"work-queue",resourceId:`prefect.work-queue.${e.id}`});return t},[o,r,a,u,n,i])}};function I({...o}){return s.jsx(k,{"data-slot":"collapsible",...o})}function $({...o}){return s.jsx(w,{"data-slot":"collapsible-trigger",...o})}function F({...o}){return s.jsx(Q,{"data-slot":"collapsible-content",...o})}export{I as C,$ as a,F as b,j as u};
//# sourceMappingURL=collapsible-B3ULeOqx.js.map
