import{r}from"./vendor-tanstack-BpuysabX.js";const o=t=>{const[e,s]=r.useState(!1);return r.useEffect(()=>{t.current&&s(t.current.scrollWidth>t.current.clientWidth)},[t]),e};export{o as u};
//# sourceMappingURL=use-is-overflowing-Cq93g4co.js.map
