import{d as m,ae as f,S as d,g as s,j as _,G as v,h as b,c as o,o as t,i as l,n as h,a as k,F as y,m as r,cG as C,cH as g,k as x,cI as S}from"./index-CWv_Pcr2.js";import{u as V}from"./usePageTitle-DY9DP8zZ.js";const w=m({__name:"Variables",setup(B){const n=f(),e=()=>{a.value.refresh(),n.value?.refreshSubscriptions()},c=d(),a=s(()=>_(c.variables.getVariables)),u=s(()=>a.value.executed&&a.value.response?.length===0),i=s(()=>a.value.executed);return V(v.info.variables),(F,j)=>{const p=b("p-layout-default");return t(),o(p,{class:"variables"},{header:l(()=>[x(r(S),{onCreate:e})]),default:l(()=>[i.value?(t(),h(y,{key:0},[u.value?(t(),o(r(C),{key:0,onCreate:e})):(t(),o(r(g),{key:1,ref_key:"table",ref:n,onDelete:e,onUpdate:e},null,512))],64)):k("",!0)]),_:1})}}});export{w as default};
//# sourceMappingURL=Variables-ypDZ0qBA.js.map
