import{d as b,S as i,u as d,v as _,g as o,bH as v,h as f,c as D,a as B,o as h,i as c,k as u,m as n,cu as g,cv as C,bg as S}from"./index-CWv_Pcr2.js";import{u as w}from"./usePageTitle-DY9DP8zZ.js";const T=b({__name:"BlockView",setup(x){const s=i(),a=d(),t=_("blockDocumentId"),l=o(()=>t.value?[t.value]:null),r=v(s.blockDocuments.getBlockDocument,l),e=o(()=>r.response),m=()=>{a.push(S.blocks())},k=o(()=>e.value?`Block: ${e.value.name}`:"Block");return w(k),(y,V)=>{const p=f("p-layout-default");return e.value?(h(),D(p,{key:0,class:"block-view"},{header:c(()=>[u(n(C),{"block-document":e.value,onDelete:m},null,8,["block-document"])]),default:c(()=>[u(n(g),{"block-document":e.value},null,8,["block-document"])]),_:1})):B("",!0)}}});export{T as default};
//# sourceMappingURL=BlockView-C1e9kPNZ.js.map
