import{j as e,L as s}from"./vendor-tanstack-BpuysabX.js";import{D as a,B as t,g as i,h as r,q as n,I as o}from"./index-CREafGO4.js";const d=()=>e.jsxs("div",{className:"flex items-center justify-between",children:[e.jsx(c,{}),e.jsx(a,{id:"automations-guide",label:"Documentation"})]}),c=()=>e.jsxs("div",{className:"flex items-center gap-2",children:[e.jsx(t,{children:e.jsx(i,{children:e.jsx(r,{className:"text-xl font-semibold",children:"Automations"})})}),e.jsx(s,{to:"/automations/create","aria-label":"create automation",children:e.jsx(n,{size:"icon",className:"size-7",variant:"outline","aria-label":"create automation",children:e.jsx(o,{id:"Plus",className:"size-4"})})})]});export{d as A};
//# sourceMappingURL=automations-header-48H3dmAW.js.map
