import{j as r}from"./vendor-tanstack-BpuysabX.js";import{s}from"./index-CREafGO4.js";import{R as n}from"./route-error-state-hDLBBOXq.js";import{o as i,e as m}from"./vendor-forms-Buh7U8Ay.js";import"./vendor-radix-DIu41Ss_.js";import"./vendor-recharts-C2XeLr4O.js";import"./vendor-date-C1t37jsh.js";i({queueTab:m(["Details","Upcoming Runs","Runs"]).default("Upcoming Runs")});const j=function({error:o,reset:t}){const e=s(o,"Failed to load work queue");if(e.type!=="server-error"&&e.type!=="client-error")throw o;return r.jsxs("div",{className:"flex flex-col gap-4",children:[r.jsx("div",{children:r.jsx("h1",{className:"text-2xl font-semibold",children:"Work Queue"})}),r.jsx(n,{error:e,onRetry:t})]})};export{j as errorComponent};
//# sourceMappingURL=work-pool_._workPoolName.queue._workQueueName-CWFJ7PHJ.js.map
