import{y as m,j as t}from"./vendor-tanstack-BpuysabX.js";import{cG as p,e0 as r}from"./index-CREafGO4.js";import{D as i}from"./deployment-action-header-DZalyt8q.js";import{D as s}from"./deployment-form-vRFt_wiQ.js";import"./vendor-radix-DIu41Ss_.js";import"./vendor-recharts-C2XeLr4O.js";import"./vendor-forms-Buh7U8Ay.js";import"./vendor-date-C1t37jsh.js";import"./schema-form-input-string-format-datetime-CtefOb8F.js";import"./zod-CtcMvSO9.js";import"./use-debounce-callback-Bybo1_Zg.js";import"./timezone-select-DpL0avEm.js";import"./dropdown-menu-DZExFL3b.js";import"./form-DC8L1tSC.js";import"./tags-input-Bd-BicQ9.js";const n=({id:e})=>{const{data:o}=m(p(e));return t.jsxs("div",{className:"flex flex-col gap-4",children:[t.jsx(i,{deployment:o,action:"Duplicate"}),t.jsx(s,{deployment:o,mode:"duplicate"})]})},S=function(){const{id:o}=r.useParams();return t.jsx(n,{id:o})};export{S as component};
//# sourceMappingURL=deployment_._id.duplicate-EmE1IMKd.js.map
