import{j as e}from"./vendor-tanstack-BpuysabX.js";import{S as a,a as o,b as c,c as n,e as l}from"./index-CREafGO4.js";const d=({defaultValue:s,value:t,onSelect:r})=>e.jsxs(a,{defaultValue:s,value:t,onValueChange:r,children:[e.jsx(o,{"aria-label":"Flow run sort order",className:"w-full",children:e.jsx(c,{placeholder:"Sort by"})}),e.jsxs(n,{children:[e.jsx(l,{value:"START_TIME_DESC",children:"Newest to oldest"}),e.jsx(l,{value:"START_TIME_ASC",children:"Oldest to newest"}),e.jsx(l,{value:"NAME_ASC",children:"A to Z"}),e.jsx(l,{value:"NAME_DESC",children:"Z to A"})]})]});export{d as S};
//# sourceMappingURL=sort-filter-DRiHC-7-.js.map
